;*****************************************************************         
;*                                                               *
;* Dieses kleine Demonstrationsprogramm zeigt die Funktion von   *
;* Unterprogrammaufrufen, Interrupts (INT) und non-maskable      *
;* Interrupts (NMI).                                             *
;* Ansonsten hat das Programm keinen tieferen Sinn. In einer     *
;* Endlosschleife wird der Inhalt des X-Registers  um eine       *
;* Stelle nach rechts und der Inhalt des Y-Registers um eine     *
;* Stelle nach links geschoben, der Inhalt des ACCCUs ohne Carry *
;* um 8 erhht und das I-Bit auf 0 gesetzt, um eine stndige     *
;* Bereitschaft fr einen INT zu gewhrleisten.                  *
;* Erfolgt ein INT, so wird das X-REG um 1 erhht, bei einem NMI *
;* wird das Y-REG inkremetiert.                                  *
;*                                                               *
;*****************************************************************

;*****************************************************************

         ;********************************************
         ; Hauptprogramm loop                        *
         ;                                           *
         ;********************************************

         .org $0300   ; Hauptprogramm startet auf Adresse 0300h

         lda #$00     ; ACCU auf 00h
         ldx #$40     ; X-REG auf 40h 
         ldy #$70     ; Y-REG auf 70h
loop:    cli          ; ermgliche IRQ
         pha          ; sicher ACCU auf Stack
         txa          ; X-REG in ACCU
         ror a        ; shift ACCU rechts
         tax          ; ACCU in X-REG
         tya          ; Y-REG in ACCU
         rol a        ; shift ACCU links
         tay          ; ACCU in Y-REG
         pla          ; alter ACCU-Inhalt vom Stack
         clc          ; lsche Carry-Bit 
         jsr submain  ; springe in Unterprogramm
         jmp loop     ; bis zum Erbrechen
         

         ;********************************************
         ; Unterprogramm submain                     *
         ; wird aus Hauptprogramm loop aufgerufen    *
         ;                                           *
         ;********************************************
submain: adc #08      ; erhhe ACCU um 8
         rts          ; zurck zum Hauptprogramm


;*****************************************************************
         .org $0400  


         ;********************************************
         ;                                           *
         ; Interrupt-Routine                         *
         ;                                           *
         ;********************************************

int:     jsr subint   ; erhhe X-register um 1
         rti          ; zurck zum Hauptprogramm

         ;********************************************
         ; Unterprogramm subint                      *
         ; wird aus Interruptroutine int aufgerufen  *
         ;                                           *
         ;********************************************

subint:  inx          ; erhhe X-register um 1
         rts          ; zurck zur INT-Routine


;*****************************************************************

         .org $0500
         ;********************************************
         ;                                           *
         ; NMI-Routine                               *
         ;                                           *
         ;********************************************

nmi:     jsr subnmi   ;
         rti          ; zurck zum Hauptprogramm

         ;********************************************
         ; Unterprogramm subnmni                     *
         ; wird aus Interruptroutine nmi aufgerufen  *
         ;                                           *
         ;********************************************
         
subnmi:  iny          ; erhhe Y-register um 1
         rts          ; zurck zur NMI-Routine

;*****************************************************************

.end
